import numpy as np
import nkpack as nk

def interaction_matrix(N, K, type="roll"):
    output = None
    if K == 0:
        output = np.eye(N, dtype=int)
    elif type == "block":
        tmp = np.eye(N, dtype=int)
        tmp = tmp.repeat(K+1, axis=0)
        tmp = tmp.repeat(K+1, axis=1)
        output = tmp[0:N, 0:N]
    elif type == "local":
        tmp = [1]*(K+1) + [0]*(N-K-1)
        tmp = [np.roll(tmp, z) for z in range(N)]
        tmp = np.array(tmp)
        output = tmp.transpose()
    elif type == "random":
        tmp = nk.interaction_matrix(N, K, "random")
        output = tmp
    elif type == "centralized":
        tmp = np.eye(N, dtype=int)
        tmp[:,0:K] = 1
        tmp[0:K,K] = 1
        output = tmp
    elif type == "dependent":
        tmp = np.eye(N, dtype=int)
        tmp[:,0:K] = 1
        tmp[0:K,K] = 1
        tmp = np.flip(tmp)
        output = np.transpose(tmp)
    elif type == "hierarchical":
        tmp = np.eye(N, dtype=int)
        for i in range(N):
            if np.sum(tmp) < N*(K+1):
                tmp[i:N,i] = 1
            if np.sum(tmp) > N*(K+1):
                idx = np.sum(tmp) - N*(K+1)
                tmp[(N-idx):N,i] = 0
        output = tmp
    return output


